@@ -19,6 +19,19 @@
 #include <stdlib.h> /* For strtoul */
 #include <string.h> /* For memset and friends */
 
+/*
+ * MSVC 8.0 started to mark many standard C library functions depreciated
+ * including the *printf family and others. Tell it to shut up.
+ * (_MSC_VER is 1200 for VC6, 1300 or 1310 for vc7.net, 1400 for 8.0)
+ */
+#if defined(_MSC_VER)
+#   pragma warning(disable:4244)
+#   if _MSC_VER >= 1400
+#	pragma warning(disable:4267)
+#	pragma warning(disable:4996)
+#   endif
+#endif
+
 /*
  * Used to tag functions that are only to be visible within the module being
  * built and not outside it (where this is supported by the linker).
@@ -102,7 +115,7 @@ MODULE_SCOPE int Tpool_Init(Tcl_Interp *interp);
  */
 
 #define TCL_CMD(a,b,c) \
-  if (Tcl_CreateObjCommand((a),(b),(c),(ClientData)NULL, NULL) == NULL) \
+  if (Tcl_CreateObjCommand((a),(b),(c),NULL, NULL) == NULL) \
     return TCL_ERROR
 
 #define OPT_CMP(a,b) \
@@ -148,9 +161,9 @@ typedef struct {
     ((void (*)(Tcl_Interp *, Tcl_Obj *))((&(tclStubsPtr->tcl_PkgProvideEx))[574]))(interp, Tcl_NewStringObj(msg, -1)): \
     ((void (*)(Tcl_Interp *, const char *))((&(tclStubsPtr->tcl_PkgProvideEx))[66]))(interp, msg))
 /* TIP #337 */
-# undef Tcl_BackgroundError
-# define Tcl_BackgroundError(interp) ((threadTclVersion>85)? \
-    ((void (*)(Tcl_Interp *, int))((&(tclStubsPtr->tcl_PkgProvideEx))[609]))(interp, TCL_ERROR): \
+# undef Tcl_BackgroundException
+# define Tcl_BackgroundException(interp, result) ((threadTclVersion>85)? \
+    ((void (*)(Tcl_Interp *, int))((&(tclStubsPtr->tcl_PkgProvideEx))[609]))(interp, result): \
     ((void (*)(Tcl_Interp *))((&(tclStubsPtr->tcl_PkgProvideEx))[76]))(interp))
 #elif !TCL_MINIMUM_VERSION(8,6)
   /* 8.5, 8.4, or less - Emulate access to the error-line information */
@@ -172,6 +185,9 @@ typedef struct {
 # define Tcl_NewIntObj(value) ((threadTclVersion>86)? \
   ((Tcl_Obj * (*)(Tcl_WideInt))((&(tclStubsPtr->tcl_PkgProvideEx))[488]))((int)(value)): \
   ((Tcl_Obj * (*)(int))((&(tclStubsPtr->tcl_PkgProvideEx))[52]))(value))
+# undef Tcl_GetUnicodeFromObj
+# define Tcl_GetUnicodeFromObj ((((&(tclStubsPtr->tcl_PkgProvideEx))[378]) != ((&(tclStubsPtr->tcl_PkgProvideEx))[434])) ? \
+  ((void (*)(Tcl_Obj *, int *))((&(tclStubsPtr->tcl_PkgProvideEx))[434])) : ((void (*)(Tcl_Obj *, int *)) NULL))
 #endif
 
 #endif /* _TCL_THREAD_INT_H_ */
