@@ -12,6 +12,28 @@
 #include "threadSvCmd.h"
 #include "threadSvListCmd.h"
 
+#if defined(USE_TCL_STUBS)
+/*  Little hack to eliminate the need for "tclInt.h" here:
+    Just copy a small portion of TclIntStubs, just
+    enough to make it work */
+typedef struct TclIntStubs {
+    int magic;
+    void *hooks;
+    void (*dummy[34]) (void); /* dummy entries 0-33, not used */
+    int (*tclGetIntForIndex) (Tcl_Interp *interp, Tcl_Obj *objPtr, int endValue, int *indexPtr); /* 34 */
+} TclIntStubs;
+extern const struct TclIntStubs *tclIntStubsPtr;
+
+# undef Tcl_GetIntForIndex
+# define Tcl_GetIntForIndex(interp, obj, max, ptr) ((threadTclVersion>86)? \
+    ((int (*)(Tcl_Interp*,  Tcl_Obj *, int, int*))((&(tclStubsPtr->tcl_PkgProvideEx))[645]))((interp), (obj), (max), (ptr)): \
+	tclIntStubsPtr->tclGetIntForIndex((interp), (obj), (max), (ptr)))
+#elif TCL_MINOR_VERSION < 7
+extern int TclGetIntForIndex(Tcl_Interp*,  Tcl_Obj *, int, int*);
+# define Tcl_GetIntForIndex TclGetIntForIndex
+#endif
+
+
 /*
  * Implementation of list commands for shared variables.
  * Most of the standard Tcl list commands are implemented.
@@ -35,15 +57,6 @@ static Tcl_ObjCmdProc SvLrangeObjCmd;    /* lrange      */
 static Tcl_ObjCmdProc SvLsearchObjCmd;   /* lsearch     */
 static Tcl_ObjCmdProc SvLsetObjCmd;      /* lset        */
 
-/*
- * These two are copied verbatim from the tclUtil.c
- * since not found in the public stubs table.
- * I was just too lazy to rewrite them from scratch.
- */
-
-static int SvCheckBadOctal(Tcl_Interp*, const char *);
-static int SvGetIntForIndex(Tcl_Interp*,  Tcl_Obj *, int, int*);
-
 /*
  * Inefficient list duplicator function which,
  * however, produces deep list copies, unlike
@@ -133,13 +146,14 @@ Sv_RegisterListCommands(void)
  */
 
 static int
-SvLpopObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
-    int ret, off, llen, index = 0, iarg = 0;
+SvLpopObjCmd (
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
+    int ret, off, llen, iarg = 0;
+    int index = 0;
     Tcl_Obj *elPtr = NULL;
     Container *svObj = (Container*)arg;
 
@@ -165,12 +179,12 @@ SvLpopObjCmd (arg, interp, objc, objv)
         goto cmd_err;
     }
     if (iarg) {
-        ret = SvGetIntForIndex(interp, objv[iarg], llen-1, &index);
+        ret = Tcl_GetIntForIndex(interp, objv[iarg], llen-1, &index);
         if (ret != TCL_OK) {
             goto cmd_err;
         }
     }
-    if (index < 0 || index >= llen) {
+    if ((index < 0) || (index >= llen)) {
         goto cmd_ok; /* Ignore out-of bounds, like Tcl does */
     }
     ret = Tcl_ListObjIndex(interp, svObj->tclObj, index, &elPtr);
@@ -212,13 +226,14 @@ SvLpopObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLpushObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
-    int off, ret, flg, llen, index = 0;
+SvLpushObjCmd (
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
+    int off, ret, flg, llen;
+    int index = 0;
     Tcl_Obj *args[1];
     Container *svObj = (Container*)arg;
 
@@ -242,7 +257,7 @@ SvLpushObjCmd (arg, interp, objc, objv)
         goto cmd_err;
     }
     if ((objc - off) == 2) {
-        ret = SvGetIntForIndex(interp, objv[off+1], llen, &index);
+        ret = Tcl_GetIntForIndex(interp, objv[off+1], llen, &index);
         if (ret != TCL_OK) {
             goto cmd_err;
         }
@@ -284,12 +299,12 @@ SvLpushObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLappendObjCmd(arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLappendObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     int i, ret, flg, off;
     Tcl_Obj *dup;
     Container *svObj = (Container*)arg;
@@ -344,15 +359,16 @@ SvLappendObjCmd(arg, interp, objc, objv)
  */
 
 static int
-SvLreplaceObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLreplaceObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     const char *firstArg;
     size_t argLen;
-    int ret, off, llen, first, last, ndel, nargs, i, j;
+    int ret, off, llen, ndel, nargs, i, j;
+    int first, last;
     Tcl_Obj **args = NULL;
     Container *svObj = (Container*)arg;
 
@@ -374,11 +390,11 @@ SvLreplaceObjCmd (arg, interp, objc, objv)
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off], llen-1, &first);
+    ret = Tcl_GetIntForIndex(interp, objv[off], llen-1, &first);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off+1], llen-1, &last);
+    ret = Tcl_GetIntForIndex(interp, objv[off+1], llen-1, &last);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
@@ -443,13 +459,14 @@ SvLreplaceObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLrangeObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
-    int ret, off, llen, first, last, nargs, i, j;
+SvLrangeObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
+    int ret, off, llen, nargs, j;
+    int first, last, i;
     Tcl_Obj **elPtrs, **args;
     Container *svObj = (Container*)arg;
 
@@ -471,11 +488,11 @@ SvLrangeObjCmd (arg, interp, objc, objv)
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off], llen-1, &first);
+    ret = Tcl_GetIntForIndex(interp, objv[off], llen-1, &first);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off+1], llen-1, &last);
+    ret = Tcl_GetIntForIndex(interp, objv[off+1], llen-1, &last);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
@@ -524,13 +541,14 @@ SvLrangeObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLinsertObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
-    int off, ret, flg, llen, nargs, index = 0, i, j;
+SvLinsertObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
+    int off, ret, flg, llen, nargs, i, j;
+    int index = 0;
     Tcl_Obj **args;
     Container *svObj = (Container*)arg;
 
@@ -553,7 +571,7 @@ SvLinsertObjCmd (arg, interp, objc, objv)
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off], llen, &index);
+    ret = Tcl_GetIntForIndex(interp, objv[off], llen, &index);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
@@ -563,7 +581,7 @@ SvLinsertObjCmd (arg, interp, objc, objv)
         index = llen;
     }
 
-    nargs = objc - (off + 1);
+    nargs = objc - off - 1;
     args  = (Tcl_Obj**)ckalloc(nargs * sizeof(Tcl_Obj*));
     for (i = off + 1, j = 0; i < objc; i++, j++) {
          args[j] = Sv_DuplicateObj(objv[i]);
@@ -603,12 +621,12 @@ SvLinsertObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLlengthObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLlengthObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     int llen, off, ret;
     Container *svObj = (Container*)arg;
 
@@ -652,12 +670,12 @@ SvLlengthObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLsearchObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLsearchObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     size_t length;
     int ret, off, listc, mode, imode, ipatt, index, match, i;
     const char *patBytes;
@@ -709,7 +727,7 @@ SvLsearchObjCmd (arg, interp, objc, objv)
         match = 0;
         switch (mode) {
         case LS_GLOB:
-            match = Tcl_StringMatch(Tcl_GetString(listv[i]), patBytes);
+            match = Tcl_StringCaseMatch(Tcl_GetString(listv[i]), patBytes, 0);
             break;
 
         case LS_EXACT: {
@@ -758,14 +776,15 @@ SvLsearchObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLindexObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLindexObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     Tcl_Obj **elPtrs;
-    int ret, off, llen, index;
+    int ret, off, llen;
+    int index;
     Container *svObj = (Container*)arg;
 
     /*
@@ -786,11 +805,11 @@ SvLindexObjCmd (arg, interp, objc, objv)
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    ret = SvGetIntForIndex(interp, objv[off], llen-1, &index);
+    ret = Tcl_GetIntForIndex(interp, objv[off], llen-1, &index);
     if (ret != TCL_OK) {
         goto cmd_err;
     }
-    if (index >= 0 && index < llen) {
+    if ((index >= 0) && (index < llen)) {
         Tcl_SetObjResult(interp, Sv_DuplicateObj(elPtrs[index]));
     }
 
@@ -818,12 +837,12 @@ SvLindexObjCmd (arg, interp, objc, objv)
  */
 
 static int
-SvLsetObjCmd (arg, interp, objc, objv)
-    ClientData arg;
-    Tcl_Interp *interp;
-    int objc;
-    Tcl_Obj *const objv[];
-{
+SvLsetObjCmd(
+    ClientData arg,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[]
+) {
     Tcl_Obj *lPtr;
     int ret, argc, off;
     Container *svObj = (Container*)arg;
@@ -882,10 +901,10 @@ SvLsetObjCmd (arg, interp, objc, objv)
  */
 
 static void
-DupListObjShared(srcPtr, copyPtr)
-    Tcl_Obj *srcPtr;            /* Object with internal rep to copy. */
-    Tcl_Obj *copyPtr;           /* Object with internal rep to set. */
-{
+DupListObjShared(
+    Tcl_Obj *srcPtr,           /* Object with internal rep to copy. */
+    Tcl_Obj *copyPtr           /* Object with internal rep to set. */
+) {
     int i, llen;
     Tcl_Obj *elObj, **newObjList;
 
@@ -908,117 +927,6 @@ DupListObjShared(srcPtr, copyPtr)
     ckfree((char*)newObjList);
 }
 
-/*
- *-----------------------------------------------------------------------------
- *
- * SvCheckBadOctal --
- *
- *  Exact copy from the TclCheckBadOctal found in tclUtil.c
- *  since this is not in the stubs table.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-SvCheckBadOctal(interp, value)
-    Tcl_Interp *interp;     /* Interpreter to use for error reporting.
-                             * If NULL, then no error message is left
-                             * after errors. */
-    const char *value;      /* String to check. */
-{
-    register const char *p = value;
-
-    /*
-     * A frequent mistake is invalid octal values due to an unwanted
-     * leading zero. Try to generate a meaningful error message.
-     */
-
-    while (isspace((unsigned char)(*p))) { /* INTL: ISO space. */
-        p++;
-    }
-    if (*p == '+' || *p == '-') {
-        p++;
-    }
-    if (*p == '0') {
-        while (isdigit((unsigned char)(*p))) { /* INTL: digit. */
-            p++;
-        }
-        while (isspace((unsigned char)(*p))) { /* INTL: ISO space. */
-            p++;
-        }
-        if (*p == '\0') {
-            /* Reached end of string */
-            if (interp != NULL) {
-                Tcl_AppendResult(interp, " (looks like invalid octal number)",
-                        (char *) NULL);
-            }
-            return 1;
-        }
-    }
-    return 0;
-}
-
-/*
- *-----------------------------------------------------------------------------
- *
- * SvGetIntForIndex --
- *
- *  Exact copy from the TclGetIntForIndex found in tclUtil.c
- *  since this is not in the stubs table.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-SvGetIntForIndex(interp, objPtr, endValue, indexPtr)
-    Tcl_Interp *interp;     /* Interpreter to use for error reporting.
-                             * If NULL, then no error message is left
-                             * after errors. */
-    Tcl_Obj *objPtr;        /* Points to an object containing either
-                             * "end" or an integer. */
-    int endValue;           /* The value to be stored at "indexPtr" if
-                             * "objPtr" holds "end". */
-    int *indexPtr;          /* Location filled in with an integer
-                             * representing an index. */
-{
-    const char *bytes;
-    size_t length;
-    int offset;
-
-    bytes = Tcl_GetString(objPtr);
-    length = objPtr->length;
-
-    if ((*bytes != 'e')
-        || (strncmp(bytes, "end",((length > 3) ? 3 : length)) != 0)) {
-        if (Tcl_GetIntFromObj(NULL, objPtr, &offset) != TCL_OK) {
-            goto intforindex_error;
-        }
-        *indexPtr = offset;
-        return TCL_OK;
-    }
-    if (length <= 3) {
-        *indexPtr = endValue;
-    } else if (bytes[3] == '-') {
-        /*
-         * This is our limited string expression evaluator
-         */
-        if (Tcl_GetInt(interp, bytes+3, &offset) != TCL_OK) {
-            return TCL_ERROR;
-        }
-        *indexPtr = endValue + offset;
-    } else {
-  intforindex_error:
-        if (interp != NULL) {
-            Tcl_ResetResult(interp);
-            Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), "bad index \"",
-                    bytes, "\": must be integer or end?-integer?",(char*)NULL);
-            SvCheckBadOctal(interp, bytes);
-        }
-        return TCL_ERROR;
-    }
-    return TCL_OK;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1035,14 +943,15 @@ SvGetIntForIndex(interp, objPtr, endValue, indexPtr)
  */
 
 static Tcl_Obj*
-SvLsetFlat(interp, listPtr, indexCount, indexArray, valuePtr)
-     Tcl_Interp *interp;     /* Tcl interpreter */
-     Tcl_Obj *listPtr;       /* Pointer to the list being modified */
-     int indexCount;         /* Number of index args */
-     Tcl_Obj **indexArray;
-     Tcl_Obj *valuePtr;      /* Value arg to 'lset' */
-{
-    int elemCount, index, result, i;
+SvLsetFlat(
+     Tcl_Interp *interp,    /* Tcl interpreter */
+     Tcl_Obj *listPtr,      /* Pointer to the list being modified */
+     int indexCount,        /* Number of index args */
+     Tcl_Obj **indexArray,
+     Tcl_Obj *valuePtr      /* Value arg to 'lset' */
+) {
+    int elemCount, result, i;
+    int index;
     Tcl_Obj **elemPtrs, *chainPtr, *subListPtr;
 
     /*
@@ -1098,7 +1007,7 @@ SvLsetFlat(interp, listPtr, indexCount, indexArray, valuePtr)
          * Determine the index of the requested element.
          */
 
-        result = SvGetIntForIndex(interp, indexArray[i], elemCount-1, &index);
+        result = Tcl_GetIntForIndex(interp, indexArray[i], elemCount-1, &index);
         if (result != TCL_OK) {
             break;
         }
@@ -1107,7 +1016,7 @@ SvLsetFlat(interp, listPtr, indexCount, indexArray, valuePtr)
          * Check that the index is in range.
          */
 
-        if (index < 0 || index >= elemCount) {
+        if ((index < 0) || (index >= elemCount)) {
             Tcl_SetObjResult(interp,
                              Tcl_NewStringObj("list index out of range", -1));
             result = TCL_ERROR;
@@ -1118,7 +1027,7 @@ SvLsetFlat(interp, listPtr, indexCount, indexArray, valuePtr)
          * Break the loop after extracting the innermost sublist
          */
 
-        if (i >= (indexCount - 1)) {
+        if (i + 1 >= indexCount) {
             result = TCL_OK;
             break;
         }
